/* Calcite, kernel/src/initfs.c
 * Copyright 2025 Benji Dial
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include <initfs.h>

static const uint8_t *initfs_start;
static uint64_t initfs_length;

void set_initfs(const uint8_t *start, uint64_t length) {
  initfs_start = start;
  initfs_length = length;
}

static uint64_t decode_octal(const char *from) {
  uint64_t value = 0;
  while (*from != '\0') {
    value = value * 8 + *from - '0';
    ++from;
  }
  return value;
}

void look_up_initfs_file(
  const char *path, const uint8_t **start_out, uint64_t *length_out) {

  int path_len = 0;
  while (path[path_len] != '\0')
    ++path_len;

  uint64_t offset = 0;

  while (1) {

    if (offset + 512 > initfs_length) {
      *start_out = 0;
      return;
    }

    uint64_t length =
      decode_octal((const char *)(initfs_start + offset + 0x7c));

    int found_it = 1;

    for (int i = 0; i < path_len; ++i)
      if (initfs_start[offset + i] != path[i]) {
        found_it = 0;
        break;
      }

    if (found_it) {
      *start_out = initfs_start + offset + 512;
      *length_out = length;
      return;
    }

    offset += 512 + ((length - 1) / 512 + 1) * 512;

  }

}
